@@ -23,19 +23,10 @@ Copyright_License {
 
 #include "Blackboard/DeviceBlackboard.hpp"
 #include "Protection.hpp"
-#include "UtilsSystem.hpp"
-#include "Asset.hpp"
 #include "Device/All.hpp"
-#include "Math/Constants.h"
-#include "GlideSolvers/GlidePolar.hpp"
 #include "Simulator.hpp"
-#include "OS/Clock.hpp"
 
-#include <limits.h>
-
-#ifdef WIN32
-#include <windows.h>
-#endif
+#include <algorithm>
 
 /**
  * Initializes the DeviceBlackboard
@@ -51,12 +42,14 @@ DeviceBlackboard::DeviceBlackboard()
   gps_info.date_time_utc = BrokenDateTime::NowUTC();
   gps_info.time = fixed(gps_info.date_time_utc.GetSecondOfDay());
 
-  for (unsigned i = 0; i < NUMDEV; ++i)
-    per_device_data[i] = gps_info;
+  std::fill_n(per_device_data, unsigned(NUMDEV), gps_info);
 
   real_data = simulator_data = replay_data = gps_info;
 
   simulator.Init(simulator_data);
+
+  real_clock.Reset();
+  replay_clock.Reset();
 }
 
 /**
@@ -74,7 +67,7 @@ DeviceBlackboard::SetStartupLocation(const GeoPoint &loc, const fixed alt)
   if (Calculated().flight.flying)
     return;
 
-  for (unsigned i = 0; i < NUMDEV; ++i)
+  for (unsigned i = 0; i < unsigned(NUMDEV); ++i)
     if (!per_device_data[i].location_available)
       per_device_data[i].SetFakeLocation(loc, alt);
 
@@ -209,7 +202,7 @@ DeviceBlackboard::ExpireWallClock()
     return;
 
   bool modified = false;
-  for (unsigned i = 0; i < NUMDEV; ++i) {
+  for (unsigned i = 0; i < unsigned(NUMDEV); ++i) {
     NMEAInfo &basic = per_device_data[i];
     if (!basic.alive)
       continue;
@@ -233,7 +226,7 @@ void
 DeviceBlackboard::Merge()
 {
   real_data.Reset();
-  for (unsigned i = 0; i < NUMDEV; ++i) {
+  for (unsigned i = 0; i < unsigned(NUMDEV); ++i) {
     if (!per_device_data[i].alive)
       continue;
 
@@ -242,11 +235,10 @@ DeviceBlackboard::Merge()
     real_data.Complement(per_device_data[i]);
   }
 
-  if (replay_data.alive) {
-    /* the replay may run at a higher speed; use NMEA_INFO::Time as a
-       "fake wallclock" to prevent them from expiring too quickly */
-    replay_data.clock = replay_data.time;
+  real_clock.Normalise(real_data);
+  replay_clock.Normalise(replay_data);
 
+  if (replay_data.alive) {
     replay_data.Expire();
     SetBasic() = replay_data;
   } else if (simulator_data.alive) {
