@@ -31,6 +31,7 @@ Copyright_License {
 #include "Blackboard/DeviceBlackboard.hpp"
 #include "Components.hpp"
 #include "Port/ConfiguredPort.hpp"
+#include "Port/DumpPort.hpp"
 #include "NMEA/Info.hpp"
 #include "Thread/Mutex.hpp"
 #include "Util/StringUtil.hpp"
@@ -39,7 +40,6 @@ Copyright_License {
 #include "Operation/Operation.hpp"
 #include "OS/Clock.hpp"
 #include "../Simulator.hpp"
-#include "Asset.hpp"
 #include "Input/InputQueue.hpp"
 #include "LogFile.hpp"
 #include "Job/Job.hpp"
@@ -163,6 +163,26 @@ DeviceDescriptor::GetState() const
   return PortState::FAILED;
 }
 
+bool
+DeviceDescriptor::IsDumpEnabled() const
+{
+  return port != nullptr && port->IsEnabled();
+}
+
+void
+DeviceDescriptor::DisableDump()
+{
+  if (port != nullptr)
+    port->Disable();
+}
+
+void
+DeviceDescriptor::EnableDumpTemporarily(unsigned duration_ms)
+{
+  if (port != nullptr)
+    port->EnableTemporarily(duration_ms);
+}
+
 bool
 DeviceDescriptor::ShouldReopenDriverOnTimeout() const
 {
@@ -187,7 +207,7 @@ DeviceDescriptor::CancelAsync()
 }
 
 bool
-DeviceDescriptor::OpenOnPort(Port *_port, OperationEnvironment &env)
+DeviceDescriptor::OpenOnPort(DumpPort *_port, OperationEnvironment &env)
 {
   assert(_port != NULL);
   assert(port == NULL);
@@ -388,11 +408,14 @@ DeviceDescriptor::DoOpen(OperationEnvironment &env)
     return false;
   }
 
-  if (!port->WaitConnected(env) || !OpenOnPort(port, env)) {
+  DumpPort *dump_port = new DumpPort(port);
+  dump_port->Disable();
+
+  if (!port->WaitConnected(env) || !OpenOnPort(dump_port, env)) {
     if (!env.IsCancelled())
       ++n_failures;
 
-    delete port;
+    delete dump_port;
     return false;
   }
 
@@ -674,8 +697,9 @@ DeviceDescriptor::ForwardLine(const char *line)
      any thread, and if the Port gets closed, bad things happen */
 
   if (IsNMEAOut() && port != NULL) {
-    port->Write(line);
-    port->Write("\r\n");
+    Port *p = port;
+    p->Write(line);
+    p->Write("\r\n");
   }
 }
 
@@ -802,6 +826,7 @@ DeviceDescriptor::PutVolume(unsigned volume, OperationEnvironment &env)
 
 bool
 DeviceDescriptor::PutActiveFrequency(RadioFrequency frequency,
+                                     const TCHAR *name,
                                      OperationEnvironment &env)
 {
   assert(InMainThread());
@@ -814,11 +839,12 @@ DeviceDescriptor::PutActiveFrequency(RadioFrequency frequency,
     return false;
 
   ScopeReturnDevice restore(*this, env);
-  return device->PutActiveFrequency(frequency, env);
+  return device->PutActiveFrequency(frequency, name, env);
 }
 
 bool
 DeviceDescriptor::PutStandbyFrequency(RadioFrequency frequency,
+                                      const TCHAR *name,
                                       OperationEnvironment &env)
 {
   assert(InMainThread());
@@ -831,7 +857,7 @@ DeviceDescriptor::PutStandbyFrequency(RadioFrequency frequency,
     return false;
 
   ScopeReturnDevice restore(*this, env);
-  return device->PutStandbyFrequency(frequency, env);
+  return device->PutStandbyFrequency(frequency, name, env);
 }
 
 bool
