@@ -36,8 +36,11 @@
 
 #include <algorithm>
 
-PolarShapeEditWidget::PolarShapeEditWidget(const PolarShape &_shape)
-  :shape(_shape) {}
+#include <assert.h>
+
+PolarShapeEditWidget::PolarShapeEditWidget(const PolarShape &_shape,
+                                           DataFieldListener *_listener)
+  :shape(_shape), listener(_listener) {}
 
 static void
 LoadValue(WndProperty &e, fixed value, UnitGroup unit_group)
@@ -96,20 +99,18 @@ PolarShapeEditWidget::SetPolarShape(const PolarShape &_shape)
     LoadPoint(points[i], shape[i]);
 }
 
-static void
-SetDataAccessCallback(WndProperty &e, DataField::DataAccessCallback callback)
+PixelSize
+PolarShapeEditWidget::GetMinimumSize() const
 {
-  DataField &df = *(DataField *)e.GetDataField();
-  df.SetDataAccessCallback(callback);
+  return { Layout::Scale(200),
+      PixelScalar(Layout::GetMinimumControlHeight()) };
 }
 
-void
-PolarShapeEditWidget::SetDataAccessCallback(DataField::DataAccessCallback callback)
+PixelSize
+PolarShapeEditWidget::GetMaximumSize() const
 {
-  for (unsigned i = 0; i < ARRAY_SIZE(points); ++i) {
-    ::SetDataAccessCallback(*points[i].v, callback);
-    ::SetDataAccessCallback(*points[i].w, callback);
-  }
+  return { Layout::Scale(400),
+      PixelScalar(Layout::GetMaximumControlHeight()) };
 }
 
 void
@@ -117,7 +118,7 @@ PolarShapeEditWidget::Prepare(ContainerWindow &parent, const PixelRect &_rc)
 {
   PanelWidget::Prepare(parent, _rc);
   const DialogLook &look = UIGlobals::GetDialogLook();
-  ContainerWindow &panel = *(ContainerWindow *)GetWindow();
+  ContainerWindow &panel = (ContainerWindow &)GetWindow();
 
   const unsigned width = _rc.right - _rc.left;
   const unsigned height = _rc.bottom - _rc.top;
@@ -149,7 +150,8 @@ PolarShapeEditWidget::Prepare(ContainerWindow &parent, const PixelRect &_rc)
                                   rc, 0, style);
     DataFieldFloat *df = new DataFieldFloat(_T("%.0f"), _T("%.0f %s"),
                                             fixed(0), fixed(300), fixed(0),
-                                            fixed(1), false, NULL);
+                                            fixed(1), false,
+                                            listener);
     points[i].v->SetDataField(df);
   }
 
@@ -185,7 +187,8 @@ PolarShapeEditWidget::Prepare(ContainerWindow &parent, const PixelRect &_rc)
                                   rc, 0, style);
     DataFieldFloat *df = new DataFieldFloat(_T("%.2f"), _T("%.2f %s"),
                                             min, fixed(0), fixed(0),
-                                            step, false, NULL);
+                                            step, false,
+                                            listener);
 
     points[i].w->SetDataField(df);
   }
