@@ -31,13 +31,12 @@ Copyright_License {
 #include "Screen/LargeTextWindow.hpp"
 #include "Form/DataField/Enum.hpp"
 #include "Form/DataField/Listener.hpp"
-#include "Util/StringUtil.hpp"
 #include "UIGlobals.hpp"
 #include "Language/Language.hpp"
 #include "Compiler.h"
 
-#ifdef ENABLE_OPENGL
-#include "Screen/OpenGL/Cache.hpp"
+#ifdef HAVE_TEXT_CACHE
+#include "Screen/Custom/Cache.hpp"
 #endif
 
 enum ControlIndex {
@@ -122,7 +121,7 @@ FontEditWidget::UpdatePreview()
 
   font.Load(data);
 
-#ifdef ENABLE_OPENGL
+#ifdef HAVE_TEXT_CACHE
   TextCache::Flush();
 #endif
 
@@ -159,7 +158,7 @@ FontEditWidget::Prepare(ContainerWindow &parent, const PixelRect &rc)
   LargeTextWindowStyle preview_style;
   preview_style.Border();
   LargeTextWindow *preview = new LargeTextWindow();
-  preview->Create(*(ContainerWindow *)GetWindow(), preview_rc, preview_style);
+  preview->Create((ContainerWindow &)GetWindow(), preview_rc, preview_style);
   Add(preview);
 
   Load();
