@@ -39,7 +39,7 @@
 
 #include "Task/Factory/Create.hpp"
 #include "Task/Factory/AbstractTaskFactory.hpp"
-#include "Task/Factory/TaskFactoryConstraints.hpp"
+#include "Task/Factory/Constraints.hpp"
 
 #include "Waypoint/Waypoints.hpp"
 #include "Geo/Flat/FlatBoundingBox.hpp"
@@ -47,25 +47,60 @@
 #include "Task/Stats/TaskSummary.hpp"
 #include "Task/PathSolvers/TaskDijkstraMin.hpp"
 #include "Task/PathSolvers/TaskDijkstraMax.hpp"
+#include "Task/ObservationZones/ObservationZoneClient.hpp"
+#include "Task/ObservationZones/CylinderZone.hpp"
+
+/**
+ * According to "FAI Sporting Code / Annex A to Section 3 - Gliding",
+ * 6.3.1c and 6.3.2dii, the radius of the "start/finish ring" must be
+ * subtracted from the task distance.  This flag controls whether this
+ * behaviour is enabled.
+ *
+ * Currently, it is always enabled, but at some point, we may want to
+ * make it optional.
+ */
+constexpr bool subtract_start_finish_cylinder_radius = true;
+
+/**
+ * Determine the cylinder radius if this is a CylinderZone.  If not,
+ * return -1.
+ */
+gcc_pure
+static fixed
+GetCylinderRadiusOrMinusOne(const ObservationZone &oz)
+{
+  return oz.GetShape() == ObservationZone::Shape::CYLINDER
+    ? ((const CylinderZone &)oz).GetRadius()
+    : fixed(-1);
+}
+
+/**
+ * Determine the cylinder radius if this is a CylinderZone.  If not,
+ * return -1.
+ */
+gcc_pure
+static fixed
+GetCylinderRadiusOrMinusOne(const ObservationZoneClient &p)
+{
+  return GetCylinderRadiusOrMinusOne(p.GetObservationZone());
+}
 
 OrderedTask::OrderedTask(const TaskBehaviour &tb)
   :AbstractTask(TaskType::ORDERED, tb),
-   taskpoint_start(NULL),
-   taskpoint_finish(NULL),
+   taskpoint_start(nullptr),
+   taskpoint_finish(nullptr),
    factory_mode(tb.task_type_default),
-   active_factory(NULL),
-   ordered_behaviour(tb.ordered_defaults),
-   dijkstra_min(NULL), dijkstra_max(NULL)
+   active_factory(nullptr),
+   ordered_settings(tb.ordered_defaults),
+   dijkstra_min(nullptr), dijkstra_max(nullptr)
 {
   active_factory = CreateTaskFactory(factory_mode, *this, task_behaviour);
-  active_factory->UpdateOrderedTaskBehaviour(ordered_behaviour);
-  task_advance.SetFactoryConstraints(active_factory->GetConstraints());
+  active_factory->UpdateOrderedTaskSettings(ordered_settings);
 }
 
 OrderedTask::~OrderedTask()
 {
   RemoveAllPoints();
-  ClearMatPoints();
 
   delete active_factory;
 
@@ -105,10 +140,21 @@ UpdateObservationZones(OrderedTask::OrderedTaskPointVector &points,
 }
 
 void
-OrderedTask::UpdateGeometry()
+OrderedTask::UpdateStatsGeometry()
 {
   ScanStartFinish();
 
+  stats.task_valid = CheckTask();
+  stats.has_targets = stats.task_valid && HasTargets();
+  stats.is_mat = GetFactoryType() == TaskFactoryType::MAT;
+  stats.has_optional_starts = stats.task_valid && HasOptionalStarts();
+}
+
+void
+OrderedTask::UpdateGeometry()
+{
+  UpdateStatsGeometry();
+
   if (!HasStart() || !task_points[0])
     return;
 
@@ -119,10 +165,6 @@ OrderedTask::UpdateGeometry()
   for (const auto *tp : task_points)
     tp->ScanProjection(task_projection);
 
-  // scan location of mat control points
-  for (const auto *i : GetMatPoints())
-    i->ScanProjection(task_projection);
-
   // ... and optional start points
   for (const OrderedTaskPoint *tp : optional_start_points)
     tp->ScanProjection(task_projection);
@@ -130,16 +172,12 @@ OrderedTask::UpdateGeometry()
   // projection can now be determined
   task_projection.Update();
 
-  // update OZ's for items that depend on next-point geometry 
+  // update OZ's for items that depend on next-point geometry
   UpdateObservationZones(task_points, task_projection);
-  UpdateObservationZones(SetMatPoints(), task_projection);
   UpdateObservationZones(optional_start_points, task_projection);
 
   // now that the task projection is stable, and oz is stable,
   // calculate the bounding box in projected coordinates
-  for (const auto tp : GetMatPoints())
-    tp->UpdateBoundingBox(task_projection);
-
   for (const auto tp : task_points)
     tp->UpdateBoundingBox(task_projection);
 
@@ -149,28 +187,29 @@ OrderedTask::UpdateGeometry()
   // update stats so data can be used during task construction
   /// @todo this should only be done if not flying! (currently done with has_entered)
   if (!taskpoint_start->HasEntered()) {
-    GeoPoint loc = taskpoint_start->GetLocation();
-    UpdateStatsDistances(loc, true);
+    UpdateStatsDistances(GeoPoint::Invalid(), true);
     if (HasFinish()) {
       /// @todo: call AbstractTask::update stats methods with fake state
       /// so stats are updated
     }
   }
+
+  force_full_update = true;
 }
 
 // TIMES
 
-fixed 
-OrderedTask::ScanTotalStartTime(const AircraftState &)
+fixed
+OrderedTask::ScanTotalStartTime()
 {
   if (taskpoint_start)
     return taskpoint_start->GetEnteredState().time;
 
-  return fixed(0);
+  return fixed(-1);
 }
 
-fixed 
-OrderedTask::ScanLegStartTime(const AircraftState &)
+fixed
+OrderedTask::ScanLegStartTime()
 {
   if (active_task_point)
     return task_points[active_task_point-1]->GetEnteredState().time;
@@ -180,25 +219,139 @@ OrderedTask::ScanLegStartTime(const AircraftState &)
 
 // DISTANCES
 
+inline bool
+OrderedTask::RunDijsktraMin(const GeoPoint &location)
+{
+  const unsigned task_size = TaskSize();
+  if (task_size < 2)
+    return false;
+
+  if (dijkstra_min == nullptr)
+    dijkstra_min = new TaskDijkstraMin();
+  TaskDijkstraMin &dijkstra = *dijkstra_min;
+
+  const unsigned active_index = GetActiveIndex();
+  dijkstra.SetTaskSize(task_size - active_index);
+  for (unsigned i = active_index; i != task_size; ++i) {
+    const SearchPointVector &boundary = task_points[i]->GetSearchPoints();
+    dijkstra.SetBoundary(i - active_index, boundary);
+  }
+
+  SearchPoint ac(location, task_projection);
+  if (!dijkstra.DistanceMin(ac))
+    return false;
+
+  for (unsigned i = active_index; i != task_size; ++i)
+    SetPointSearchMin(i, dijkstra.GetSolution(i - active_index));
+
+  return true;
+}
+
 inline fixed
 OrderedTask::ScanDistanceMin(const GeoPoint &location, bool full)
 {
-  if (full) {
-    if (dijkstra_min == NULL)
-      dijkstra_min = new TaskDijkstraMin();
+  if (!full && location.IsValid() && last_min_location.IsValid() &&
+      DistanceIsSignificant(location, last_min_location)) {
+    const TaskWaypoint *active = GetActiveTaskPoint();
+    if (active != nullptr) {
+      const GeoPoint &target = active->GetWaypoint().location;
+      const unsigned last_distance =
+        (unsigned)last_min_location.Distance(target);
+      const unsigned cur_distance =
+        (unsigned)location.Distance(target);
 
-    SearchPoint ac(location, task_projection);
-    if (dijkstra_min->DistanceMin(*this, ac)) {
-      for (unsigned i = GetActiveIndex(), end = TaskSize(); i != end; ++i)
-        SetPointSearchMin(i, dijkstra_min->GetSolution(i));
+      /* do the full scan only if the distance to the active task
+         point has changed by more than 5%, otherwise we don't expect
+         any relevant changes */
+      if (last_distance < 2000 || cur_distance < 2000 ||
+          last_distance * 20 >= cur_distance * 21 ||
+          cur_distance * 20 >= last_distance * 21)
+        full = true;
     }
+  }
 
+  if (full) {
+    RunDijsktraMin(location);
     last_min_location = location;
   }
 
   return taskpoint_start->ScanDistanceMin();
 }
 
+inline bool
+OrderedTask::RunDijsktraMax()
+{
+  const unsigned task_size = TaskSize();
+  if (task_size < 2)
+    return false;
+
+  if (dijkstra_max == nullptr)
+    dijkstra_max = new TaskDijkstraMax();
+  TaskDijkstraMax &dijkstra = *dijkstra_max;
+
+  const unsigned active_index = GetActiveIndex();
+  dijkstra.SetTaskSize(task_size);
+  for (unsigned i = 0; i != task_size; ++i) {
+    const SearchPointVector &boundary = i == active_index
+      /* since one can still travel further in the current sector, use
+         the full boundary here */
+      ? task_points[i]->GetBoundaryPoints()
+      : task_points[i]->GetSearchPoints();
+    dijkstra_max->SetBoundary(i, boundary);
+  }
+
+  fixed start_radius(-1), finish_radius(-1);
+  if (subtract_start_finish_cylinder_radius) {
+    /* to subtract the start/finish cylinder radius, we use only the
+       nominal points (i.e. the cylinder's center), and later replace
+       it with a point on the cylinder boundary */
+
+    if (taskpoint_start != nullptr) {
+      start_radius = GetCylinderRadiusOrMinusOne(*taskpoint_start);
+      if (positive(start_radius))
+        dijkstra.SetBoundary(0, task_points[0]->GetNominalPoints());
+    }
+
+    if (taskpoint_finish != nullptr) {
+      finish_radius = GetCylinderRadiusOrMinusOne(*taskpoint_finish);
+      if (positive(finish_radius))
+        dijkstra.SetBoundary(task_size - 1,
+                             task_points[task_size - 1]->GetNominalPoints());
+    }
+  }
+
+  if (!dijkstra_max->DistanceMax())
+    return false;
+
+  for (unsigned i = 0; i != task_size; ++i) {
+    SearchPoint solution = dijkstra.GetSolution(i);
+
+    if (i == 0 && positive(start_radius)) {
+      /* subtract start cylinder radius by finding the intersection
+         with the cylinder boundary */
+      const GeoPoint &current = taskpoint_start->GetLocation();
+      const GeoPoint &neighbour = dijkstra.GetSolution(i + 1).GetLocation();
+      GeoPoint gp = current.IntermediatePoint(neighbour, start_radius);
+      solution = SearchPoint(gp, task_projection);
+    }
+
+    if (i == task_size - 1 && positive(finish_radius)) {
+      /* subtract finish cylinder radius by finding the intersection
+         with the cylinder boundary */
+      const GeoPoint &current = taskpoint_finish->GetLocation();
+      const GeoPoint &neighbour = dijkstra.GetSolution(i - 1).GetLocation();
+      GeoPoint gp = current.IntermediatePoint(neighbour, finish_radius);
+      solution = SearchPoint(gp, task_projection);
+    }
+
+    SetPointSearchMax(i, solution);
+    if (i <= active_index)
+      set_tp_search_achieved(i, solution);
+  }
+
+  return true;
+}
+
 inline fixed
 OrderedTask::ScanDistanceMax()
 {
@@ -207,32 +360,8 @@ OrderedTask::ScanDistanceMax()
 
   assert(active_task_point < task_points.size());
 
-  // for max calculations, since one can still travel further in the
-  // sector, we pretend we are on the previous turnpoint so the
-  // search samples will contain the full boundary
-  const unsigned atp = active_task_point;
-  if (atp) {
-    active_task_point--;
-    taskpoint_start->ScanActive(*task_points[active_task_point]);
-  }
+  RunDijsktraMax();
 
-  if (dijkstra_max == NULL)
-    dijkstra_max = new TaskDijkstraMax();
-
-  if (dijkstra_max->DistanceMax(*this)) {
-    for (unsigned i = 0, active = GetActiveIndex(), end = TaskSize();
-         i != end; ++i) {
-      const SearchPoint &solution = dijkstra_max->GetSolution(i);
-      SetPointSearchMax(i, solution);
-      if (i <= active)
-        set_tp_search_achieved(i, solution);
-    }
-  }
-
-  if (atp) {
-    active_task_point = atp;
-    taskpoint_start->ScanActive(*task_points[active_task_point]);
-  }
   return taskpoint_start->ScanDistanceMax();
 }
 
@@ -246,17 +375,30 @@ OrderedTask::ScanDistanceMinMax(const GeoPoint &location, bool force,
   if (force)
     *dmax = ScanDistanceMax();
 
-  bool force_min = force || DistanceIsSignificant(location, last_min_location);
-  *dmin = ScanDistanceMin(location, force_min);
+  *dmin = ScanDistanceMin(location, force);
 }
 
 fixed
 OrderedTask::ScanDistanceNominal()
 {
-  if (taskpoint_start)
-    return taskpoint_start->ScanDistanceNominal();
+  if (taskpoint_start == nullptr)
+    return fixed(0);
 
-  return fixed(0);
+  fixed d = taskpoint_start->ScanDistanceNominal();
+
+  if (subtract_start_finish_cylinder_radius) {
+    fixed radius = GetCylinderRadiusOrMinusOne(*taskpoint_start);
+    if (positive(radius) && radius < d)
+      d -= radius;
+
+    if (taskpoint_finish != nullptr) {
+      radius = GetCylinderRadiusOrMinusOne(*taskpoint_finish);
+      if (positive(radius) && radius < d)
+        d -= radius;
+    }
+  }
+
+  return d;
 }
 
 fixed
@@ -307,29 +449,9 @@ OrderedTask::GetLastIntermediateAchieved() const
   return TaskSize() - 2;
 }
 
-bool
-OrderedTask::ShouldAddToMat(const Waypoint &mat_wp) const
-{
-  unsigned last_achieved_index = GetLastIntermediateAchieved();
-
-  // is this the same point we just achieved? then ignore it
-  if (mat_wp == GetPoint(last_achieved_index).GetWaypoint())
-  {
-    return false;
-  }
-
-  // The TP after the last achieved is already next in the task
-  // and it is not the finish so do nothing
-  if ((last_achieved_index + 1 < TaskSize() -1)
-    && mat_wp == GetPoint(last_achieved_index + 1).GetWaypoint()) {
-    return false;
-  }
-  return true;
-}
-
 // TRANSITIONS
 
-bool 
+bool
 OrderedTask::CheckTransitions(const AircraftState &state,
                               const AircraftState &state_last)
 {
@@ -389,7 +511,7 @@ OrderedTask::CheckTransitions(const AircraftState &state,
           SetActiveTaskPoint(i);
           taskpoint_start->ScanActive(*task_points[active_task_point]);
 
-          if (task_events != NULL)
+          if (task_events != nullptr)
             task_events->ActiveAdvanced(*task_points[i], i);
 
           // on sector exit, must update samples since start sector
@@ -397,12 +519,14 @@ OrderedTask::CheckTransitions(const AircraftState &state,
           full_update = true;
         }
       } else if (!last_request_armed && task_advance.NeedToArm()) {
-        if (task_events != NULL)
+        if (task_events != nullptr)
           task_events->RequestArm(*task_points[i]);
       }
     }
   }
 
+  stats.need_to_arm = task_advance.NeedToArm();
+
   taskpoint_start->ScanActive(*task_points[active_task_point]);
 
   stats.task_finished = taskpoint_finish != nullptr &&
@@ -416,7 +540,7 @@ OrderedTask::CheckTransitions(const AircraftState &state,
     taskpoint_finish->set_fai_finish_height(start_state.altitude - fixed(1000));
   }
 
-  if (task_events != NULL) {
+  if (task_events != nullptr) {
     if (stats.start.task_started && !last_started)
       task_events->TaskStart();
 
@@ -458,18 +582,6 @@ OrderedTask::CheckTransitionOptionalStart(const AircraftState &state,
   return full_update;
 }
 
-bool
-OrderedTask::CheckTransitionPointMat(OrderedTaskPoint &point,
-                                     const AircraftState &state,
-                                     const AircraftState &state_last,
-                                     const FlatBoundingBox &bb_now,
-                                     const FlatBoundingBox &bb_last)
-{
-  const bool nearby = point.BoundingBoxOverlaps(bb_now);
-
-  return nearby && point.CheckEnterTransitionMat(state, state_last);
-}
-
 bool
 OrderedTask::CheckTransitionPoint(OrderedTaskPoint &point,
                                   const AircraftState &state,
@@ -487,24 +599,24 @@ OrderedTask::CheckTransitionPoint(OrderedTaskPoint &point,
   if (nearby && point.TransitionEnter(state, state_last)) {
     transition_enter = true;
 
-    if (task_events != NULL)
+    if (task_events != nullptr)
       task_events->EnterTransition(point);
   }
-  
+
   if (nearby && point.TransitionExit(state, state_last, task_projection)) {
     transition_exit = true;
 
-    if (task_events != NULL)
+    if (task_events != nullptr)
       task_events->ExitTransition(point);
-    
+
     // detect restart
     if (is_start && last_started)
       last_started = false;
   }
-  
-  if (is_start) 
+
+  if (is_start)
     UpdateStartTransition(state, point);
-  
+
   return nearby
     ? point.UpdateSampleNear(state, task_projection)
     : point.UpdateSampleFar(state, task_projection);
@@ -512,17 +624,17 @@ OrderedTask::CheckTransitionPoint(OrderedTaskPoint &point,
 
 // ADDITIONAL FUNCTIONS
 
-bool 
+bool
 OrderedTask::UpdateIdle(const AircraftState &state,
                         const GlidePolar &glide_polar)
 {
   bool retval = AbstractTask::UpdateIdle(state, glide_polar);
 
   if (HasStart() && task_behaviour.optimise_targets_range &&
-      positive(GetOrderedTaskBehaviour().aat_min_time)) {
+      positive(GetOrderedTaskSettings().aat_min_time)) {
 
     CalcMinTarget(state, glide_polar,
-                  GetOrderedTaskBehaviour().aat_min_time + fixed(task_behaviour.optimise_targets_margin));
+                  GetOrderedTaskSettings().aat_min_time + fixed(task_behaviour.optimise_targets_margin));
 
     if (task_behaviour.optimise_targets_bearing &&
         task_points[active_task_point]->GetType() == TaskPointType::AAT) {
@@ -535,15 +647,17 @@ OrderedTask::UpdateIdle(const AircraftState &state,
     }
     retval = true;
   }
-  
+
   return retval;
 }
 
-bool 
+bool
 OrderedTask::UpdateSample(const AircraftState &state,
                           const GlidePolar &glide_polar,
-                           const bool full_update)
+                          const bool full_update)
 {
+  assert(state.location.IsValid());
+
   stats.inside_oz = active_task_point < task_points.size() &&
     task_points[active_task_point]->IsInSector(state);
 
@@ -555,8 +669,8 @@ OrderedTask::UpdateSample(const AircraftState &state,
 void
 OrderedTask::SetNeighbours(unsigned position)
 {
-  OrderedTaskPoint* prev = NULL;
-  OrderedTaskPoint* next = NULL;
+  OrderedTaskPoint* prev = nullptr;
+  OrderedTaskPoint* next = nullptr;
 
   if (!task_points[position])
     // nothing to do if this is deleted
@@ -590,15 +704,15 @@ AATPoint*
 OrderedTask::GetAATTaskPoint(unsigned TPindex) const
 {
  if (TPindex > task_points.size() - 1) {
-   return NULL;
+   return nullptr;
  }
  if (task_points[TPindex]) {
     if (task_points[TPindex]->GetType() == TaskPointType::AAT)
       return (AATPoint*) task_points[TPindex];
     else
-      return (AATPoint*)NULL;
+      return (AATPoint*)nullptr;
  }
- return NULL;
+ return nullptr;
 }
 
 bool
@@ -606,19 +720,19 @@ OrderedTask::ScanStartFinish()
 {
   /// @todo also check there are not more than one start/finish point
   if (task_points.empty()) {
-    taskpoint_start = NULL;
-    taskpoint_finish = NULL;
+    taskpoint_start = nullptr;
+    taskpoint_finish = nullptr;
     return false;
   }
 
   taskpoint_start = task_points[0]->GetType() == TaskPointType::START
     ? (StartPoint *)task_points[0]
-    : NULL;
+    : nullptr;
 
   taskpoint_finish = task_points.size() > 1 &&
     task_points[task_points.size() - 1]->GetType() == TaskPointType::FINISH
     ? (FinishPoint *)task_points[task_points.size() - 1]
-    : NULL;
+    : nullptr;
 
   return HasStart() && HasFinish();
 }
@@ -655,7 +769,6 @@ OrderedTask::Remove(const unsigned position)
   if (position)
     SetNeighbours(position - 1);
 
-  UpdateGeometry();
   return true;
 }
 
@@ -670,11 +783,10 @@ OrderedTask::RemoveOptionalStart(const unsigned position)
   if (task_points.size()>1)
     SetNeighbours(0);
 
-  UpdateGeometry();
   return true;
 }
 
-bool 
+bool
 OrderedTask::Append(const OrderedTaskPoint &new_tp)
 {
   if (/* is the new_tp allowed in this context? */
@@ -684,7 +796,7 @@ OrderedTask::Append(const OrderedTaskPoint &new_tp)
        !task_points[task_points.size() - 1]->IsSuccessorAllowed()))
     return false;
 
-  task_points.push_back(new_tp.Clone(task_behaviour, ordered_behaviour));
+  task_points.push_back(new_tp.Clone(task_behaviour, ordered_settings));
   if (task_points.size() > 1)
     SetNeighbours(task_points.size() - 2);
   else {
@@ -693,22 +805,20 @@ OrderedTask::Append(const OrderedTaskPoint &new_tp)
   }
 
   SetNeighbours(task_points.size() - 1);
-  UpdateGeometry();
   return true;
 }
 
-bool 
+bool
 OrderedTask::AppendOptionalStart(const OrderedTaskPoint &new_tp)
 {
   optional_start_points.push_back(new_tp.Clone(task_behaviour,
-                                               ordered_behaviour));
+                                               ordered_settings));
   if (task_points.size() > 1)
     SetNeighbours(0);
-  UpdateGeometry();
   return true;
 }
 
-bool 
+bool
 OrderedTask::Insert(const OrderedTaskPoint &new_tp, const unsigned position)
 {
   if (position >= task_points.size())
@@ -726,7 +836,7 @@ OrderedTask::Insert(const OrderedTaskPoint &new_tp, const unsigned position)
     active_task_point++;
 
   task_points.insert(task_points.begin() + position,
-                     new_tp.Clone(task_behaviour, ordered_behaviour));
+                     new_tp.Clone(task_behaviour, ordered_settings));
 
   if (position)
     SetNeighbours(position - 1);
@@ -734,11 +844,10 @@ OrderedTask::Insert(const OrderedTaskPoint &new_tp, const unsigned position)
   SetNeighbours(position);
   SetNeighbours(position + 1);
 
-  UpdateGeometry();
   return true;
 }
 
-bool 
+bool
 OrderedTask::Replace(const OrderedTaskPoint &new_tp, const unsigned position)
 {
   if (position >= task_points.size())
@@ -754,7 +863,7 @@ OrderedTask::Replace(const OrderedTaskPoint &new_tp, const unsigned position)
     return false;
 
   delete task_points[position];
-  task_points[position] = new_tp.Clone(task_behaviour, ordered_behaviour);
+  task_points[position] = new_tp.Clone(task_behaviour, ordered_settings);
 
   if (position)
     SetNeighbours(position - 1);
@@ -763,12 +872,11 @@ OrderedTask::Replace(const OrderedTaskPoint &new_tp, const unsigned position)
   if (position + 1 < task_points.size())
     SetNeighbours(position + 1);
 
-  UpdateGeometry();
   return true;
 }
 
 
-bool 
+bool
 OrderedTask::ReplaceOptionalStart(const OrderedTaskPoint &new_tp,
                                   const unsigned position)
 {
@@ -781,25 +889,22 @@ OrderedTask::ReplaceOptionalStart(const OrderedTaskPoint &new_tp,
 
   delete optional_start_points[position];
   optional_start_points[position] = new_tp.Clone(task_behaviour,
-                                                 ordered_behaviour);
+                                                 ordered_settings);
 
   SetNeighbours(0);
-  UpdateGeometry();
   return true;
 }
 
 
-void 
+void
 OrderedTask::SetActiveTaskPoint(unsigned index)
 {
-  if (index < task_points.size()) {
-    if (active_task_point != index)
-      task_advance.SetArmed(false);
+  if (index >= task_points.size() || index == active_task_point)
+    return;
 
-    active_task_point = index;
-  } else if (task_points.empty()) {
-    active_task_point = 0;
-  }
+  task_advance.SetArmed(false);
+  active_task_point = index;
+  force_full_update = true;
 }
 
 TaskWaypoint*
@@ -808,22 +913,21 @@ OrderedTask::GetActiveTaskPoint() const
   if (active_task_point < task_points.size())
     return task_points[active_task_point];
 
-  return NULL;
+  return nullptr;
 }
 
-bool 
+bool
 OrderedTask::IsValidTaskPoint(const int index_offset) const
 {
   unsigned index = active_task_point + index_offset;
   return (index < task_points.size());
 }
 
-
 void
 OrderedTask::GlideSolutionRemaining(const AircraftState &aircraft,
-                                      const GlidePolar &polar,
-                                      GlideResult &total,
-                                      GlideResult &leg)
+                                    const GlidePolar &polar,
+                                    GlideResult &total,
+                                    GlideResult &leg)
 {
   TaskMacCreadyRemaining tm(task_points.cbegin(), task_points.cend(),
                             active_task_point,
@@ -835,9 +939,15 @@ OrderedTask::GlideSolutionRemaining(const AircraftState &aircraft,
 void
 OrderedTask::GlideSolutionTravelled(const AircraftState &aircraft,
                                     const GlidePolar &glide_polar,
-                                      GlideResult &total,
-                                      GlideResult &leg)
+                                    GlideResult &total,
+                                    GlideResult &leg)
 {
+  if (!aircraft.location.IsValid()) {
+    total.Reset();
+    leg.Reset();
+    return;
+  }
+
   TaskMacCreadyTravelled tm(task_points.cbegin(), active_task_point,
                             task_behaviour.glide, glide_polar);
   total = tm.glide_solution(aircraft);
@@ -847,12 +957,12 @@ OrderedTask::GlideSolutionTravelled(const AircraftState &aircraft,
 void
 OrderedTask::GlideSolutionPlanned(const AircraftState &aircraft,
                                   const GlidePolar &glide_polar,
-                                    GlideResult &total,
-                                    GlideResult &leg,
-                                    DistanceStat &total_remaining_effective,
-                                    DistanceStat &leg_remaining_effective,
-                                    const GlideResult &solution_remaining_total,
-                                    const GlideResult &solution_remaining_leg)
+                                  GlideResult &total,
+                                  GlideResult &leg,
+                                  DistanceStat &total_remaining_effective,
+                                  DistanceStat &leg_remaining_effective,
+                                  const GlideResult &solution_remaining_total,
+                                  const GlideResult &solution_remaining_leg)
 {
   TaskMacCreadyTotal tm(task_points.cbegin(), task_points.cend(),
                         active_task_point,
@@ -927,7 +1037,7 @@ OrderedTask::CalcCruiseEfficiency(const AircraftState &aircraft,
   }
 }
 
-bool 
+bool
 OrderedTask::CalcEffectiveMC(const AircraftState &aircraft,
                              const GlidePolar &glide_polar,
                              fixed &val) const
@@ -949,7 +1059,7 @@ OrderedTask::CalcMinTarget(const AircraftState &aircraft,
                            const GlidePolar &glide_polar,
                            const fixed t_target)
 {
-  if (stats.distance_max > stats.distance_min) {
+  if (stats.has_targets) {
     // only perform scan if modification is possible
     const fixed t_rem = std::max(fixed(0),
                                  t_target - stats.total.time_elapsed);
@@ -964,8 +1074,7 @@ OrderedTask::CalcMinTarget(const AircraftState &aircraft,
   return fixed(0);
 }
 
-
-fixed 
+fixed
 OrderedTask::CalcGradient(const AircraftState &state) const
 {
   if (task_points.empty())
@@ -992,7 +1101,7 @@ Visit(const OrderedTask::OrderedTaskPointVector &points,
     visitor.Visit(*tp);
 }
 
-void 
+void
 OrderedTask::AcceptTaskPointVisitor(TaskPointConstVisitor& visitor) const
 {
   Visit(task_points, visitor);
@@ -1007,7 +1116,7 @@ ResetPoints(OrderedTask::OrderedTaskPointVector &points)
 
 void
 OrderedTask::Reset()
-{  
+{
   /// @todo also reset data in this class e.g. stats?
   ResetPoints(task_points);
   ResetPoints(optional_start_points);
@@ -1017,14 +1126,15 @@ OrderedTask::Reset()
   stats.start.task_started = false;
   task_advance.Reset();
   SetActiveTaskPoint(0);
+  UpdateStatsGeometry();
 }
 
-bool 
+bool
 OrderedTask::TaskStarted(bool soft) const
 {
   if (taskpoint_start) {
     // have we really started?
-    if (taskpoint_start->HasExited()) 
+    if (taskpoint_start->HasExited())
       return true;
 
     // if soft starts allowed, consider started if we progressed to next tp
@@ -1069,20 +1179,20 @@ OrderedTask::GetPointSearchPoints(unsigned tp) const
   return task_points[tp]->GetSearchPoints();
 }
 
-void 
+void
 OrderedTask::SetPointSearchMin(unsigned tp, const SearchPoint &sol)
 {
   task_points[tp]->SetSearchMin(sol);
 }
 
-void 
-OrderedTask::set_tp_search_achieved(unsigned tp, const SearchPoint &sol) 
+void
+OrderedTask::set_tp_search_achieved(unsigned tp, const SearchPoint &sol)
 {
   if (task_points[tp]->HasSampled())
     SetPointSearchMin(tp, sol);
 }
 
-void 
+void
 OrderedTask::SetPointSearchMax(unsigned tp, const SearchPoint &sol)
 {
   task_points[tp]->SetSearchMax(sol);
@@ -1120,7 +1230,7 @@ OrderedTask::HasTargets() const
   return false;
 }
 
-GeoPoint 
+GeoPoint
 OrderedTask::GetTaskCenter() const
 {
   if (!HasStart() || !task_points[0])
@@ -1129,23 +1239,24 @@ OrderedTask::GetTaskCenter() const
   return task_projection.GetCenter();
 }
 
-fixed 
+fixed
 OrderedTask::GetTaskRadius() const
-{ 
+{
   if (!HasStart() || !task_points[0])
     return fixed(0);
 
   return task_projection.ApproxRadius();
 }
 
-OrderedTask* 
+OrderedTask*
 OrderedTask::Clone(const TaskBehaviour &tb) const
 {
   OrderedTask* new_task = new OrderedTask(tb);
 
-  new_task->ordered_behaviour = ordered_behaviour;
-
   new_task->SetFactory(factory_mode);
+
+  new_task->ordered_settings = ordered_settings;
+
   for (const OrderedTaskPoint *tp : task_points)
     new_task->Append(*tp);
 
@@ -1168,7 +1279,7 @@ OrderedTask::CheckDuplicateWaypoints(Waypoints& waypoints,
       waypoints.CheckExistsOrAppend((*i)->GetWaypoint());
 
     const OrderedTaskPoint *new_tp =
-      (*i)->Clone(task_behaviour, ordered_behaviour, &wp);
+      (*i)->Clone(task_behaviour, ordered_settings, &wp);
     if (is_task)
       Replace(*new_tp, std::distance(begin, i));
     else
@@ -1189,11 +1300,11 @@ OrderedTask::Commit(const OrderedTask& that)
 {
   bool modified = false;
 
-  // change mode to that one 
+  // change mode to that one
   SetFactory(that.factory_mode);
 
   // copy across behaviour
-  SetOrderedTaskBehaviour(that.ordered_behaviour);
+  SetOrderedTaskSettings(that.ordered_settings);
 
   // remove if that task is smaller than this one
   while (TaskSize() > that.TaskSize()) {
@@ -1233,8 +1344,6 @@ OrderedTask::Commit(const OrderedTask& that)
     }
   }
 
-  mat_points.CloneFrom(that.mat_points, task_behaviour, ordered_behaviour);
-
   if (modified)
     UpdateGeometry();
     // @todo also re-scan task sample state,
@@ -1250,7 +1359,7 @@ OrderedTask::RelocateOptionalStart(const unsigned position, const Waypoint& wayp
     return false;
 
   OrderedTaskPoint *new_tp =
-    optional_start_points[position]->Clone(task_behaviour, ordered_behaviour,
+    optional_start_points[position]->Clone(task_behaviour, ordered_settings,
                                            &waypoint);
   delete optional_start_points[position];
   optional_start_points[position]= new_tp;
@@ -1264,7 +1373,7 @@ OrderedTask::Relocate(const unsigned position, const Waypoint& waypoint)
     return false;
 
   OrderedTaskPoint *new_tp = task_points[position]->Clone(task_behaviour,
-                                                  ordered_behaviour,
+                                                  ordered_settings,
                                                   &waypoint);
   bool success = Replace(*new_tp, position);
   delete new_tp;
@@ -1289,32 +1398,30 @@ OrderedTask::SetFactory(const TaskFactoryType the_factory)
 
   delete active_factory;
   active_factory = CreateTaskFactory(factory_mode, *this, task_behaviour);
-  active_factory->UpdateOrderedTaskBehaviour(ordered_behaviour);
+  active_factory->UpdateOrderedTaskSettings(ordered_settings);
 
-  task_advance.SetFactoryConstraints(active_factory->GetConstraints());
-
-  PropagateOrderedTaskBehaviour();
-}
-
-void 
-OrderedTask::SetOrderedTaskBehaviour(const OrderedTaskBehaviour& ob)
-{
-  ordered_behaviour = ob;
-
-  PropagateOrderedTaskBehaviour();
+  PropagateOrderedTaskSettings();
 }
 
 void
-OrderedTask::PropagateOrderedTaskBehaviour()
+OrderedTask::SetOrderedTaskSettings(const OrderedTaskSettings& ob)
 {
-  for (auto tp : task_points)
-    tp->SetOrderedTaskBehaviour(ordered_behaviour);
+  ordered_settings = ob;
 
-  for (auto tp : optional_start_points)
-    tp->SetOrderedTaskBehaviour(ordered_behaviour);
+  PropagateOrderedTaskSettings();
 }
 
-bool 
+void
+OrderedTask::PropagateOrderedTaskSettings()
+{
+  for (auto tp : task_points)
+    tp->SetOrderedTaskSettings(ordered_settings);
+
+  for (auto tp : optional_start_points)
+    tp->SetOrderedTaskSettings(ordered_settings);
+}
+
+bool
 OrderedTask::IsScored() const
 {
   return GetFactoryConstraints().task_scored;
@@ -1345,8 +1452,9 @@ OrderedTask::RemoveAllPoints()
 
   optional_start_points.clear();
 
-  taskpoint_start = NULL;
-  taskpoint_finish = NULL;
+  taskpoint_start = nullptr;
+  taskpoint_finish = nullptr;
+  force_full_update = true;
 }
 
 void
@@ -1355,11 +1463,11 @@ OrderedTask::Clear()
   RemoveAllPoints();
 
   Reset();
-  ordered_behaviour = task_behaviour.ordered_defaults;
-  active_factory->UpdateOrderedTaskBehaviour(ordered_behaviour);
+  ordered_settings = task_behaviour.ordered_defaults;
+  active_factory->UpdateOrderedTaskSettings(ordered_settings);
 }
 
-FlatBoundingBox 
+FlatBoundingBox
 OrderedTask::GetBoundingBox(const GeoBounds &bounds) const
 {
   if (!TaskSize()) {
@@ -1375,18 +1483,18 @@ OrderedTask::GetBoundingBox(const GeoBounds &bounds) const
                     std::min(ll.latitude, lr.latitude));
   FlatGeoPoint fmax(std::max(lr.longitude, ur.longitude),
                     std::max(ul.latitude, ur.latitude));
-  // note +/- 1 to ensure rounding keeps bb valid 
+  // note +/- 1 to ensure rounding keeps bb valid
   fmin.longitude -= 1; fmin.latitude -= 1;
   fmax.longitude += 1; fmax.latitude += 1;
   return FlatBoundingBox (fmin, fmax);
 }
 
-void 
+void
 OrderedTask::RotateOptionalStarts()
 {
   if (!TaskSize())
     return;
-  if (!optional_start_points.size()) 
+  if (!optional_start_points.size())
     return;
 
   SelectOptionalStart(0);
@@ -1439,19 +1547,3 @@ OrderedTask::UpdateSummary(TaskSummary& ordered_summary) const
     ordered_summary.update(stats.total.remaining.GetDistance(),
                            stats.total.planned.GetDistance());
 }
-
-void
-OrderedTask::FillMatPoints(const Waypoints &wps, bool update_geometry)
-{
-  ClearMatPoints();
-  GeoPoint center = TaskSize() > 0
-    ? GetPoint(0).GetLocation()
-    : task_projection.GetCenter();
-
-  if (GetFactoryType() == TaskFactoryType::MAT) {
-    mat_points.FillMatPoints(wps, GetFactory(), center);
-
-    if (update_geometry)
-      UpdateGeometry();
-  }
-}
