@@ -202,25 +202,6 @@ public:
    */
   bool UpdateAutoMC(const AircraftState& state_now, const fixed fallback_mc);
 
-  /**
-   * Scans all the MAT points to see if any have been entered.
-   * If point is entered and not next unachieved point,
-   * Inserts the point into the active ordered task, and calls Commit().
-   * Assumes the protected task manager already holds an Exclusive Lease
-   * on the Task Manager.
-   * Should be called before CheckTransitions for a MAT task.
-   *
-   * @return true if point is inserted into task
-   */
-  bool ScanInsertMatPoints(const AircraftState &state,
-                           const AircraftState &state_last);
-
-  /**
-   * loads the Mat points vector to the current task if it is a Mat
-   * @param waypoints the list of active waypoints
-   */
-  void FillMatPoints(const Waypoints &waypoints);
-
   /**
    * Accessor for statistics of active task
    *
@@ -440,11 +421,11 @@ public:
   void SetTaskBehaviour(const TaskBehaviour& behaviour);
 
   /** 
-   * Retrieve the OrderedTaskBehaviour used by the OrderedTask
+   * Retrieve the #OrderedTaskSettings used by the OrderedTask
    * 
-   * @return OrderedTaskBehaviour reference
+   * @return #OrderedTaskSettings reference
    */
-  void SetOrderedTaskBehaviour(const OrderedTaskBehaviour &otb);
+  void SetOrderedTaskSettings(const OrderedTaskSettings &otb);
 
   /** 
    * Retrieve task behaviour
