@@ -67,6 +67,8 @@ DigitEntry::Create(ContainerWindow &parent, const PixelRect &rc,
     digit.value = 0;
   }
 
+  max_width = rc.right - rc.left;
+
   CalculateLayout();
 
   PaintWindow::Create(parent, rc, style);
@@ -198,6 +200,19 @@ DigitEntry::CalculateLayout()
     digit.left = last_right;
     last_right = digit.right = digit.left + value_width;
   }
+
+  /* check if width of entry form exceeds available width and adjust
+     column widths appropiately */
+  if (last_right > max_width) {
+    unsigned width_adjust = (last_right - max_width) / length + 1;
+    last_right = 0;
+    for (unsigned i = 0; i < length; ++i) {
+      Column &digit = columns[i];
+      PixelScalar value_width = digit.right - digit.left;
+      digit.left = last_right;
+      last_right = digit.right = digit.left + value_width - width_adjust;
+    }
+  }
 }
 
 int
@@ -713,7 +728,7 @@ DigitEntry::OnPaint(Canvas &canvas)
 {
   assert(cursor < length);
 
-  const bool focused = HasFocus();
+  const bool focused = HasCursorKeys() && HasFocus();
 
   if (HaveClipping())
     canvas.Clear(look.background_color);
