@@ -24,14 +24,13 @@ Copyright_License {
 #include "Form/Form.hpp"
 #include "Time/PeriodClock.hpp"
 #include "Asset.hpp"
-#include "Interface.hpp"
 #include "Screen/Canvas.hpp"
 #include "Screen/SingleWindow.hpp"
 #include "Screen/Layout.hpp"
 #include "Screen/Key.h"
-#include "Util/StringUtil.hpp"
 #include "Util/Macros.hpp"
 #include "Look/DialogLook.hpp"
+#include "Event/Globals.hpp"
 
 #ifndef USE_GDI
 #include "Screen/Custom/Reference.hpp"
@@ -42,16 +41,14 @@ Copyright_License {
 #endif
 
 #ifdef ANDROID
-#include "Android/Main.hpp"
-#include "Event/Android/Event.hpp"
+#include "Event/Shared/Event.hpp"
 #include "Event/Android/Loop.hpp"
 #elif defined(ENABLE_SDL)
 #include "Event/SDL/Event.hpp"
 #include "Event/SDL/Loop.hpp"
-#elif defined(USE_EGL)
-#include "Event/EGL/Globals.hpp"
-#include "Event/EGL/Event.hpp"
-#include "Event/EGL/Loop.hpp"
+#elif defined(USE_CONSOLE) || defined(NON_INTERACTIVE)
+#include "Event/Shared/Event.hpp"
+#include "Event/Console/Loop.hpp"
 #elif defined(USE_GDI)
 #include "Event/GDI/Event.hpp"
 #include "Event/GDI/Loop.hpp"
@@ -228,6 +225,16 @@ WndForm::OnMouseMove(PixelScalar x, PixelScalar y, unsigned keys)
     if (new_position.top < 0)
       new_position.Offset(0, -new_position.top);
 
+#ifdef USE_MEMORY_CANVAS
+    /* the RasterCanvas class doesn't clip negative window positions
+       properly, therefore we avoid this problem at this stage */
+    if (new_position.left < 0)
+      new_position.left = 0;
+
+    if (new_position.top < 0)
+      new_position.top = 0;
+#endif
+
     Move(new_position.left, new_position.top);
 
     return true;
@@ -354,7 +361,6 @@ WndForm::ShowModal()
 {
   AssertNoneLocked();
 
-#define OPENCLOSESUPPRESSTIME 500
 #ifndef USE_GDI
   ContainerWindow *root = GetRootOwner();
   WindowReference old_focus_reference = root->GetFocusedWindowReference();
@@ -376,11 +382,7 @@ WndForm::ShowModal()
 #ifdef USE_GDI
   oldFocusHwnd = ::GetFocus();
 #endif /* USE_GDI */
-  SetFocus();
-  if (default_focus)
-    default_focus->SetFocus();
-  else
-    client_area.FocusFirstControl();
+  SetDefaultFocus();
 
   bool hastimed = false;
 
@@ -390,12 +392,10 @@ WndForm::ShowModal()
   main_window.Refresh();
 #endif
 
-#if defined(ANDROID) || defined(USE_EGL)
+#if defined(ANDROID) || defined(USE_CONSOLE) || defined(ENABLE_SDL) || defined(NON_INTERACTIVE)
   EventLoop loop(*event_queue, main_window);
-#elif defined(ENABLE_SDL)
-  EventLoop loop(main_window);
 #else
-  DialogEventLoop loop(*this);
+  DialogEventLoop loop(*event_queue, *this);
 #endif
   Event event;
 
@@ -418,12 +418,12 @@ WndForm::ShowModal()
     }
 
     if (event.IsKeyDown()) {
-      if (key_down_function &&
+      if (
 #ifdef USE_GDI
           IdentifyDescendant(event.msg.hwnd) &&
 #endif
           !CheckSpecialKey(this, event) &&
-          key_down_function(event.GetKeyCode()))
+          OnAnyKeyDown(event.GetKeyCode()))
         continue;
 
 #ifdef ENABLE_SDL
@@ -472,6 +472,14 @@ WndForm::ShowModal()
         continue;
       }
 #endif
+
+#ifdef USE_LINUX_INPUT
+      if (event.GetKeyCode() == KEY_POWER) {
+        /* the Kobo power button closes the modal dialog */
+        modal_result = mrCancel;
+        continue;
+      }
+#endif
     }
 
     if (event.IsCharacter() && character_function &&
@@ -503,9 +511,9 @@ WndForm::OnPaint(Canvas &canvas)
   gcc_unused const bool is_active = main_window.IsTopDialog(*this);
 
 #ifdef ENABLE_OPENGL
-  if (!IsMaximised() && is_active) {
+  if (!IsDithered() && !IsMaximised() && is_active) {
     /* draw a shade around the current dialog to emphasise it */
-    GLEnable blend(GL_BLEND);
+    const GLBlend blend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnableClientState(GL_COLOR_ARRAY);
 
     const PixelRect rc = GetClientRect();
@@ -563,7 +571,16 @@ WndForm::OnPaint(Canvas &canvas)
   PixelRect rcClient = GetClientRect();
 
   // Draw the borders
-  canvas.DrawRaisedEdge(rcClient);
+  if (!IsMaximised()) {
+#ifndef USE_GDI
+    if (IsDithered())
+      canvas.DrawOutlineRectangle(rcClient.left, rcClient.top,
+                                  rcClient.right - 1, rcClient.bottom - 1,
+                                  COLOR_BLACK);
+    else
+#endif
+      canvas.DrawRaisedEdge(rcClient);
+  }
 
   if (!caption.empty()) {
     // Set the colors
@@ -575,32 +592,37 @@ WndForm::OnPaint(Canvas &canvas)
     // JMW todo add here icons?
 
 #ifdef EYE_CANDY
-    canvas.SetBackgroundTransparent();
-    canvas.Stretch(title_rect.left, title_rect.top,
-                   title_rect.right - title_rect.left,
-                   title_rect.bottom - title_rect.top,
-                   look.caption.background_bitmap);
+    if (!IsDithered()) {
+      canvas.SetBackgroundTransparent();
+      canvas.Stretch(title_rect.left, title_rect.top,
+                     title_rect.right - title_rect.left,
+                     title_rect.bottom - title_rect.top,
+                     look.caption.background_bitmap);
 
-    // Draw titlebar text
-    canvas.DrawText(title_rect.left + Layout::FastScale(2), title_rect.top,
-                    caption.c_str());
-#else
-    canvas.SetBackgroundColor(is_active
-                              ? look.caption.background_color
-                              : look.caption.inactive_background_color);
-    canvas.DrawOpaqueText(title_rect.left + Layout::FastScale(2),
-                          title_rect.top, title_rect, caption.c_str());
+      // Draw titlebar text
+      canvas.DrawText(title_rect.left + Layout::FastScale(2), title_rect.top,
+                      caption.c_str());
+    } else {
+#endif
+      canvas.SetBackgroundColor(is_active
+                                ? look.caption.background_color
+                                : look.caption.inactive_background_color);
+      canvas.DrawOpaqueText(title_rect.left + Layout::FastScale(2),
+                            title_rect.top, title_rect, caption.c_str());
+#ifdef EYE_CANDY
+    }
 #endif
   }
 
   if (dragging) {
 #ifdef ENABLE_OPENGL
-    GLEnable blend(GL_BLEND);
-    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    const GLBlend blend(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     canvas.DrawFilledRectangle(0, 0, canvas.GetWidth(), canvas.GetHeight(),
                                COLOR_YELLOW.WithAlpha(80));
 #elif defined(USE_GDI)
-    ::InvertRect(canvas, &title_rect);
+    canvas.InvertRectangle(title_rect);
+#else
+    canvas.InvertRectangle(GetClientRect());
 #endif
   }
 }
@@ -619,28 +641,47 @@ WndForm::SetCaption(const TCHAR *_caption)
   }
 }
 
-#ifdef ANDROID
 void
-WndForm::ReinitialiseLayout()
+WndForm::ReinitialiseLayout(const PixelRect &parent_rc)
 {
-  const SingleWindow &main_window = GetMainWindow();
+  const unsigned parent_width = parent_rc.right - parent_rc.left;
+  const unsigned parent_height = parent_rc.bottom - parent_rc.top;
 
-  if (main_window.GetWidth() < GetWidth() ||
-      main_window.GetHeight() < GetHeight()) {
-    // close dialog, it's creator may want to create a new layout
-    modal_result = mrChangeLayout;
+  if (parent_width < GetWidth() || parent_height < GetHeight()) {
   } else {
     // reposition dialog to fit into TopWindow
-    PixelScalar left = GetLeft();
-    PixelScalar top = GetTop();
+    PixelRect rc = GetPosition();
 
-    if (GetRight() > (PixelScalar) main_window.GetWidth())
-      left = main_window.GetWidth() - GetWidth();
-    if (GetBottom() > (PixelScalar) main_window.GetHeight())
-      top = main_window.GetHeight() - GetHeight();
+    if (rc.right > (PixelScalar)parent_width)
+      rc.left = parent_width - (rc.right - rc.left);
+    if (rc.bottom > (PixelScalar)parent_height)
+      rc.top = parent_height - (rc.bottom - rc.top);
 
-    if (left != GetLeft() || top != GetTop())
-      Move(left, top);
+#ifdef USE_MEMORY_CANVAS
+    /* the RasterCanvas class doesn't clip negative window positions
+       properly, therefore we avoid this problem at this stage */
+    if (rc.left < 0)
+      rc.left = 0;
+    if (rc.top < 0)
+      rc.top = 0;
+#endif
+
+    Move(rc.left, rc.top);
   }
 }
-#endif
+
+void
+WndForm::SetDefaultFocus()
+{
+  SetFocus();
+  if (default_focus)
+    default_focus->SetFocus();
+  else
+    client_area.FocusFirstControl();
+}
+
+bool
+WndForm::OnAnyKeyDown(unsigned key_code)
+{
+  return key_down_function && key_down_function(key_code);
+}
