@@ -25,7 +25,6 @@ Copyright_License {
 #include "Language/Language.hpp"
 
 #include <algorithm>
-#include <stdio.h>
 #include <tchar.h>
 
 using namespace InfoBoxFactory;
@@ -34,7 +33,7 @@ void
 InfoBoxSettings::Panel::Clear()
 {
   name.clear();
-  std::fill(contents, contents + MAX_CONTENTS, InfoBoxFactory::MIN_TYPE_VAL);
+  std::fill_n(contents, MAX_CONTENTS, InfoBoxFactory::MIN_TYPE_VAL);
 }
 
 bool
@@ -52,7 +51,7 @@ InfoBoxSettings::SetDefaults()
 {
   use_final_glide = false;
 
-  geometry = Geometry::TOP_4_BOTTOM_4;
+  geometry = Geometry::SPLIT_8;
 
   inverse = false;
   use_colors = true;
