@@ -171,8 +171,8 @@ InputEvents::sub_PanCursor(int dx, int dy)
     return;
 
   RasterPoint pt = projection.GetScreenOrigin();
-  pt.x -= dx * projection.GetScreenWidth() / 4;
-  pt.y -= dy * projection.GetScreenHeight() / 4;
+  pt.x -= dx * int(projection.GetScreenWidth()) / 4;
+  pt.y -= dy * int(projection.GetScreenHeight()) / 4;
   map_window->SetLocation(projection.ScreenToGeo(pt));
 
   map_window->QuickRedraw();
