@@ -22,10 +22,10 @@ Copyright_License {
 */
 
 #include "MarkerLook.hpp"
-#include "resource.h"
+#include "Resources.hpp"
 
 void
 MarkerLook::Initialise()
 {
-  icon.Load(IDB_MARK, IDB_MARK_HD);
+  icon.LoadResource(IDB_MARK, IDB_MARK_HD);
 }
