@@ -39,7 +39,6 @@ Copyright_License {
 struct Look;
 struct GestureLook;
 class Logger;
-class SingleWindow;
 
 class OffsetHistory
 {
@@ -62,6 +61,17 @@ class GlueMapWindow : public MapWindow {
 
   unsigned idle_robin;
 
+#ifdef ENABLE_OPENGL
+  /**
+   * A timer that triggers a redraw periodically until all data files
+   * (terrain, topography, ...) have been loaded / updated.  This is
+   * necessary if there is no valid GPS input, and no other reason to
+   * redraw is present.  This timer will cease automatically once all
+   * data is loaded, i.e. Idle() returned false.
+   */
+  WindowTimer data_timer;
+#endif
+
   PeriodClock mouse_down_clock;
 
   enum DragMode {
@@ -135,6 +145,7 @@ class GlueMapWindow : public MapWindow {
 
 public:
   GlueMapWindow(const Look &look);
+  virtual ~GlueMapWindow();
 
   void SetLogger(Logger *_logger) {
     logger = _logger;
@@ -184,8 +195,11 @@ protected:
   virtual void DrawThermalEstimate(Canvas &canvas) const override;
   virtual void RenderTrail(Canvas &canvas,
                            const RasterPoint aircraft_pos) override;
+  virtual void RenderTrackBearing(Canvas &canvas,
+                                  const RasterPoint aircraft_pos) override;
 
   /* virtual methods from class Window */
+  virtual void OnDestroy() override;
   virtual bool OnMouseDouble(PixelScalar x, PixelScalar y) override;
   virtual bool OnMouseMove(PixelScalar x, PixelScalar y, unsigned keys) override;
   virtual bool OnMouseDown(PixelScalar x, PixelScalar y) override;
@@ -225,7 +239,7 @@ private:
   void DrawVario(Canvas &canvas, const PixelRect &rc) const;
   void DrawStallRatio(Canvas &canvas, const PixelRect &rc) const;
 
-  void SwitchZoomClimb(bool circling);
+  void SwitchZoomClimb();
 
   void SaveDisplayModeScales();
 
@@ -242,6 +256,13 @@ private:
 
 public:
   void UpdateMapScale();
+
+  /**
+   * Restore the map scale from MapSettings::cruise_scale or
+   * MapSettings::circling_scale.
+   */
+  void RestoreMapScale();
+
   void UpdateDisplayMode();
   void SetMapScale(fixed scale);
 
