@@ -68,6 +68,13 @@ Double(fixed a)
   return a * 2;
 }
 
+constexpr
+static inline fixed
+Quadruple(fixed a)
+{
+  return a * 4;
+}
+
 gcc_const
 inline fixed rsqrt(fixed a) {
   // not fast
@@ -109,18 +116,10 @@ inline fixed accurate_half_sin(fixed a) {
 #include <type_traits>
 #include <climits>
 
-#ifdef HAVE_BOOST
-#include <boost/cstdint.hpp>
-#else
 #include <stdint.h>
-#endif
 
 class fixed
 {
-#ifdef HAVE_BOOST
-  typedef boost::int64_t int64_t;
-  typedef boost::uint64_t uint64_t;
-#endif
   typedef uint64_t uvalue_t;
 
 public:
@@ -341,6 +340,11 @@ public:
     return fixed(internal(), m_nVal << 1);
   }
 
+  constexpr
+  fixed Quadruple() const {
+    return fixed(internal(), m_nVal << 2);
+  }
+
   gcc_pure
   fixed trunc() const {
     value_t x = m_nVal;
@@ -953,6 +957,13 @@ Double(fixed a)
   return a.Double();
 }
 
+constexpr
+static inline fixed
+Quadruple(fixed a)
+{
+  return a.Quadruple();
+}
+
 #endif
 
 /**
