@@ -26,9 +26,15 @@ Copyright_License {
 
 #include "Screen/Point.hpp"
 
-#ifdef ENABLE_OPENGL
+#ifdef USE_MEMORY_CANVAS
+#include "Screen/Memory/Buffer.hpp"
+#include "Screen/Memory/PixelTraits.hpp"
+#endif
+
+#ifdef ANDROID
 #include "Util/tstring.hpp"
 #include "Screen/OpenGL/Surface.hpp"
+#include "ResourceId.hpp"
 #endif
 
 #ifdef USE_GDI
@@ -37,23 +43,26 @@ Copyright_License {
 
 #include <assert.h>
 #include <tchar.h>
-#include <stdint.h>
 
+class ResourceId;
 class UncompressedImage;
+template<typename T> struct ConstBuffer;
 
 #ifdef ENABLE_OPENGL
 class GLTexture;
+#elif !defined(USE_GDI)
+#ifdef GREYSCALE
+using BitmapPixelTraits = GreyscalePixelTraits;
+#else
+using BitmapPixelTraits = BGRAPixelTraits;
 #endif
-
-#ifdef ENABLE_SDL
-struct SDL_Surface;
 #endif
 
 /**
  * An image loaded from storage.
  */
-class Bitmap
-#ifdef ENABLE_OPENGL
+class Bitmap final
+#ifdef ANDROID
              : private GLSurfaceListener
 #endif
 {
@@ -72,35 +81,42 @@ public:
   };
 
 protected:
-#ifdef ENABLE_OPENGL
+#ifdef ANDROID
   /** resource id */
-  unsigned id;
+  ResourceId id;
   /** filename for external images (id=0) */
   tstring pathName;
+#endif
+
+#ifdef ENABLE_OPENGL
   GLTexture *texture;
   PixelSize size;
-#elif defined(ENABLE_SDL)
-  SDL_Surface *surface;
+
+  bool interpolation;
+#elif defined(USE_MEMORY_CANVAS)
+  WritableImageBuffer<BitmapPixelTraits> buffer;
 #else
   HBITMAP bitmap;
 #endif
 
 public:
 #ifdef ENABLE_OPENGL
-  Bitmap():id(0), texture(NULL) {}
-  explicit Bitmap(unsigned id):texture(NULL) {
-    Load(id);
-  }
-#elif defined(ENABLE_SDL)
-  Bitmap():surface(NULL) {}
-  explicit Bitmap(unsigned id):surface(NULL) {
-    Load(id);
-  }
+  Bitmap()
+    :
+#ifdef ANDROID
+    id(ResourceId::Null()),
+#endif
+    texture(NULL), interpolation(false) {}
+#elif defined(USE_MEMORY_CANVAS)
+  constexpr Bitmap():buffer(WritableImageBuffer<BitmapPixelTraits>::Empty()) {}
 #else
   Bitmap():bitmap(NULL) {}
-  explicit Bitmap(unsigned id):bitmap(NULL) {
-    Load(id);
-  }
+#endif
+
+  explicit Bitmap(ResourceId id);
+
+#if !defined(USE_GDI) && !defined(ANDROID)
+  Bitmap(ConstBuffer<void> buffer);
 #endif
 
   ~Bitmap() {
@@ -109,40 +125,52 @@ public:
 
   Bitmap(const Bitmap &other) = delete;
   Bitmap &operator=(const Bitmap &other) = delete;
-
 public:
   bool IsDefined() const {
 #ifdef ENABLE_OPENGL
     return texture != NULL;
-#elif defined(ENABLE_SDL)
-    return surface != NULL;
+#elif defined(USE_MEMORY_CANVAS)
+    return buffer.data != nullptr;
 #else
     return bitmap != NULL;
 #endif
   }
 
 #ifdef ENABLE_OPENGL
-  UPixelScalar GetWidth() const {
+  unsigned GetWidth() const {
     return size.cx;
   }
 
-  UPixelScalar GetHeight() const {
+  unsigned GetHeight() const {
     return size.cy;
   }
+#elif defined(USE_MEMORY_CANVAS)
+  unsigned GetWidth() const {
+    return buffer.width;
+  }
+
+  unsigned GetHeight() const {
+    return buffer.height;
+  }
 #endif
 
-  bool Load(const UncompressedImage &uncompressed, Type type=Type::STANDARD);
-
-#ifdef ENABLE_SDL
-  bool Load(SDL_Surface *_surface, Type type=Type::STANDARD);
+#ifdef ENABLE_OPENGL
+  void EnableInterpolation();
+#else
+  void EnableInterpolation() {}
 #endif
 
-  bool Load(unsigned id, Type type=Type::STANDARD);
+#if !defined(USE_GDI) && !defined(ANDROID)
+  bool Load(const UncompressedImage &uncompressed, Type type=Type::STANDARD);
+  bool Load(ConstBuffer<void> buffer, Type type=Type::STANDARD);
+#endif
+
+  bool Load(ResourceId id, Type type=Type::STANDARD);
 
   /**
    * Load a bitmap and stretch it by the specified zoom factor.
    */
-  bool LoadStretch(unsigned id, unsigned zoom);
+  bool LoadStretch(ResourceId id, unsigned zoom);
 
   bool LoadFile(const TCHAR *path);
 
@@ -154,9 +182,9 @@ public:
   GLTexture *GetNative() const {
     return texture;
   }
-#elif defined(ENABLE_SDL)
-  SDL_Surface* GetNative() const {
-    return surface;
+#elif defined(USE_MEMORY_CANVAS)
+  ConstImageBuffer<BitmapPixelTraits> GetNative() const {
+    return buffer;
   }
 #else
   HBITMAP GetNative() const {
@@ -166,7 +194,7 @@ public:
   }
 #endif
 
-#ifdef ENABLE_OPENGL
+#ifdef ANDROID
 private:
   /**
    * Load the texture again after the OpenGL surface has been
