@@ -23,19 +23,19 @@ Copyright_License {
 
 #include "Screen/OpenGL/Init.hpp"
 #include "Screen/OpenGL/Debug.hpp"
-#include "Screen/OpenGL/Cache.hpp"
 #include "Screen/OpenGL/Globals.hpp"
 #include "Screen/OpenGL/Extension.hpp"
 #include "Screen/OpenGL/Features.hpp"
 #include "Screen/OpenGL/Shapes.hpp"
 #include "FBO.hpp"
+#include "Screen/Custom/Cache.hpp"
 
 #ifdef ANDROID
 #include "Android/Main.hpp"
 #include "Android/NativeView.hpp"
 #endif
 
-#if defined(HAVE_EGL) && defined(ANDROID)
+#if defined(HAVE_DYNAMIC_EGL)
 #include "EGL.hpp"
 #endif
 
@@ -189,7 +189,7 @@ CheckStencil()
 void
 OpenGL::SetupContext()
 {
-#if defined(HAVE_EGL) && defined(ANDROID)
+#if defined(HAVE_DYNAMIC_EGL)
   egl = EGLInit();
 #endif
 
