@@ -33,18 +33,14 @@ Copyright_License {
 #include "Terrain/RasterWeather.hpp"
 #include "Input/InputEvents.hpp"
 #include "Input/InputQueue.hpp"
-#include "Geo/Geoid.hpp"
-#include "Dialogs/XML.hpp"
 #include "Dialogs/Dialogs.h"
 #include "Dialogs/dlgSimulatorPrompt.hpp"
 #include "Language/LanguageGlue.hpp"
 #include "Language/Language.hpp"
 #include "Protection.hpp"
 #include "LogFile.hpp"
-#include "Math/FastMath.h"
 #include "UtilsSystem.hpp"
 #include "FLARM/Glue.hpp"
-#include "MapSettings.hpp"
 #include "Logger/Logger.hpp"
 #include "Logger/NMEALogger.hpp"
 #include "Logger/GlueFlightLogger.hpp"
@@ -58,16 +54,12 @@ Copyright_License {
 #include "Topography/TopographyGlue.hpp"
 #include "Audio/VarioGlue.hpp"
 #include "Screen/Busy.hpp"
-#include "Polar/PolarGlue.hpp"
-#include "Polar/Polar.hpp"
 #include "CommandLine.hpp"
 #include "MainWindow.hpp"
-#include "Look/Look.hpp"
-#include "Look/Fonts.hpp"
-#include "resource.h"
 #include "Computer/GlideComputer.hpp"
 #include "Computer/GlideComputerInterface.hpp"
 #include "Computer/Events.hpp"
+#include "Monitor/AllMonitors.hpp"
 #include "StatusMessage.hpp"
 #include "MergeThread.hpp"
 #include "CalculationThread.hpp"
@@ -76,7 +68,7 @@ Copyright_License {
 #include "IO/FileCache.hpp"
 #include "Net/DownloadManager.hpp"
 #include "Hardware/AltairControl.hpp"
-#include "Hardware/Display.hpp"
+#include "Hardware/DisplayDPI.hpp"
 #include "Hardware/DisplayGlue.hpp"
 #include "Compiler.h"
 #include "NMEA/Aircraft.hpp"
@@ -91,7 +83,6 @@ Copyright_License {
 #include "Task/TaskManager.hpp"
 #include "Task/ProtectedTaskManager.hpp"
 #include "Engine/Task/Ordered/OrderedTask.hpp"
-#include "GlideSolvers/GlidePolar.hpp"
 #include "Operation/VerboseOperationEnvironment.hpp"
 #include "PageActions.hpp"
 #include "Weather/Features.hpp"
@@ -113,6 +104,7 @@ Copyright_License {
 static Markers *marks;
 static TaskManager *task_manager;
 static GlideComputerEvents *glide_computer_events;
+static AllMonitors *all_monitors;
 static GlideComputerTaskEvents *task_events;
 
 static bool
@@ -159,7 +151,6 @@ AfterStartup()
       way_points.Optimise();
     }
 
-    defaultTask->FillMatPoints(way_points);
     protected_task_manager->TaskCommit(*defaultTask);
     delete defaultTask;
   }
@@ -202,7 +193,8 @@ Startup()
   TopWindowStyle style;
   if (CommandLine::full_screen)
     style.FullScreen();
-  if (CommandLine::resizable)
+
+  if (!IsWindowsCE())
     style.Resizable();
 
   MainWindow *const main_window = CommonInterface::main_window =
@@ -213,11 +205,11 @@ Startup()
 
 #ifdef ENABLE_OPENGL
   LogFormat("OpenGL: "
-#ifdef HAVE_EGL
+#ifdef HAVE_DYNAMIC_EGL
             "egl=%d "
 #endif
             "npot=%d vbo=%d fbo=%d stencil=%#x",
-#ifdef HAVE_EGL
+#ifdef HAVE_DYNAMIC_EGL
              OpenGL::egl,
 #endif
              OpenGL::texture_non_power_of_two,
@@ -231,12 +223,6 @@ Startup()
 #ifdef SIMULATOR_AVAILABLE
   // prompt for simulator if not set by command line argument "-simulator" or "-fly"
   if (!sim_set_in_cmd_line_flag) {
-    DialogLook white_look;
-    white_look.Initialise(Fonts::map_bold, Fonts::map, Fonts::map_label,
-                          Fonts::map_bold, Fonts::map_bold, Fonts::map_bold);
-    white_look.SetBackgroundColor(COLOR_WHITE);
-    SetXMLDialogLook(white_look);
-
     SimulatorPromptResult result = dlgSimulatorPromptShowModal();
     switch (result) {
     case SPR_QUIT:
@@ -253,8 +239,6 @@ Startup()
   }
 #endif
 
-  SetXMLDialogLook(main_window->GetLook().dialog);
-
   CommonInterface::SetSystemSettings().SetDefaults();
   CommonInterface::SetComputerSettings().SetDefaults();
   CommonInterface::SetUISettings().SetDefaults();
@@ -273,6 +257,7 @@ Startup()
   CreateDataPath();
 
   Display::LoadOrientation(operation);
+  main_window->CheckResize();
 
   main_window->InitialiseConfigured();
 
@@ -335,9 +320,6 @@ Startup()
 
   replay = new Replay(logger, *protected_task_manager);
 
-  // Load the EGM96 geoid data
-  EGM96::Load();
-
   GlidePolar &gp = CommonInterface::SetComputerSettings().polar.glide_polar_task;
   gp = GlidePolar(fixed(0));
   gp.SetMC(computer_settings.task.safety_mc);
@@ -448,7 +430,9 @@ Startup()
   glide_computer_events->Reset();
   live_blackboard.AddListener(*glide_computer_events);
 
-  if (computer_settings.logger.enable_flight_logger) {
+  all_monitors = new AllMonitors();
+
+  if (!is_simulator() && computer_settings.logger.enable_flight_logger) {
     flight_logger = new GlueFlightLogger(live_blackboard);
     LocalPath(path, _T("flights.log"));
     flight_logger->SetPath(path);
@@ -524,6 +508,7 @@ Shutdown()
   delete flight_logger;
   flight_logger = NULL;
 
+  delete all_monitors;
   live_blackboard.RemoveListener(*glide_computer_events);
   delete glide_computer_events;
 
@@ -533,10 +518,6 @@ Shutdown()
   operation.SetText(_("Shutdown, saving profile..."));
   Profile::Save();
 
-  // Stop sound
-
-  AudioVarioGlue::Deinitialise();
-
   operation.SetText(_("Shutdown, please wait..."));
 
   // Stop threads
@@ -572,6 +553,9 @@ Shutdown()
   LogFormat("delete MapWindow");
   main_window->Deinitialise();
 
+  // Stop sound
+  AudioVarioGlue::Deinitialise();
+
   // Save the task for the next time
   operation.SetText(_("Shutdown, saving task..."));
 
@@ -630,15 +614,12 @@ Shutdown()
   LogFormat("Close Progress Dialog");
   operation.Hide();
 
-  // Clear the EGM96 database
-  EGM96::Close();
-
   delete glide_computer;
   delete task_events;
   delete logger;
 
   // Clear airspace database
-  airspace_database.clear();
+  airspace_database.Clear();
 
   // Destroy FlarmNet records
   DeinitTrafficGlobals();
