@@ -24,7 +24,6 @@
 #define MAPTASKMANAGER_H
 
 struct Waypoint;
-class OrderedTask;
 
 /**
  * Appends wp to current Ordered task and activates the ordered task if
@@ -48,15 +47,6 @@ namespace MapTaskManager
   TaskEditResult AppendToTask(const Waypoint &wp);
   TaskEditResult InsertInTask(const Waypoint &wp);
 
-  /**
-   * inserts the waypoint into the Ordered Task
-   * It is up to the caller to decide whether to commit the new task or not.
-   * Task manager must be in Ordered Mode when called
-   * @param task.  Pointer to an ordered Mat task
-   * @param waypoint.  The wp to insert into the task before the active tp
-   */
-  TaskEditResult InsertInMatProForma(OrderedTask &task,
-                                     const Waypoint &waypoint);
   TaskEditResult ReplaceInTask(const Waypoint &wp);
   TaskEditResult RemoveFromTask(const Waypoint &wp);
 
@@ -66,14 +56,6 @@ namespace MapTaskManager
    * else returns -1
    */
   int GetIndexInTask(const Waypoint &wp);
-
-  /**
-   * is this wp an unachieved task point?
-   * @param waypoint.  the wp being tested
-   * @return.  -1 or last index in ordered task where the wp is an
-   * unachieved intermediate point.  -1 if mode != MODE_ORDDERED
-   */
-  int GetUnachievedIndexInTask(const Waypoint &waypoint);
 };
 
 #endif
